module Apscanner
  class SnmpManager
    SNMP_OID = {
      wlc: {
        name: '1.3.6.1.4.1.9.9.513.1.1.1.1.5',
        mac: '1.3.6.1.4.1.9.9.513.1.2.3.1.1',
        clients: '1.3.6.1.4.1.9.9.513.1.1.1.1.54'
      },
      meraki: {
        system: '1.3.6.1.4.1.29671.1.1.4.1',
        mac: '1.3.6.1.4.1.29671.1.1.4.1.1',
        name: '1.3.6.1.4.1.29671.1.1.4.1.2',
        status: '1.3.6.1.4.1.29671.1.1.4.1.3',
        clients: '1.3.6.1.4.1.29671.1.1.4.1.5'
      }
    }.freeze

    def initialize(sensor)
      @sensor = sensor
    end

    def run
      response = request_by_type
      return [] if empty?(response)
      SnmpParser.new(response, @sensor.type).run
    end

    private

    def empty?(response)
      response.empty? || response.values.uniq == [{}]
    end

    def manager
      SNMP4EM::Manager.new(
        host: @sensor.ip_address,
        community: @sensor.community || 'public',
        version: :SNMPv2c,
        port: @sensor.port || 161,
        retries: 2
      )
    end

    def oids(type)
      oids = SNMP_OID[type].values
      type == :wlc ? oids : [oids.first]
    end

    def discover
      response = {}
      SNMP_OID.keys.each do |type|
        response = request(oids(type))
        unless empty?(response)
          @sensor.type = type
          break
        end
      end
      response
    end

    def request_by_type
      if SNMP_OID.keys.include?(@sensor.type)
        request(oids(@sensor.type))
      else
        discover
      end
    end

    def request(oids)
      response = {}
      EM.run do
        req = manager.bulkwalk(oids)
        req.errback do |error|
          Apscanner.logger.error(error)
          EM.stop
        end

        req.callback do |resp|
          Apscanner.logger.debug("#{@sensor.ip_address}: #{resp}")
          response = resp
          EM.stop
        end
      end
      response
    end   
  end
end
