module Apscanner
  class SnmpParser
    def initialize(response, type)
      @response = response
      @type = type
      @oids = SnmpManager::SNMP_OID[type]
    end

    def run
      send("parse_#{@type}")
    end

    private

    def parse_wlc
      parse
    end

    def parse_meraki
      @response = {
        @oids[:mac] => sys.select { |k, _| k.include?("#{@oids[:mac]}.") },
        @oids[:name] => sys.select { |k, _| k.include?("#{@oids[:name]}.") },
        @oids[:clients] => sys.select { |k, _| k.include?("#{@oids[:clients]}.") },
        @oids[:status] => sys.select { |k, _| k.include?("#{@oids[:status]}.") }
      }
      parse
    end

    def parse
      macs.map do |oid, value|
        ap_oid = parse_ap_oid(oid)
        AccessPoint.new(
          name: names[@oids[:name] + '.' + ap_oid],
          mac_address: parse_mac(value),
          clients: clients[@oids[:clients] + '.' + ap_oid]
        ) if status_up?(ap_oid)
      end.compact
    end

    def parse_ap_oid(oid)
      oid.gsub("#{@oids[:mac]}.", '')
    end

    def macs
      @response[@oids[:mac]]
    end

    def names
      @response[@oids[:name]]
    end

    def clients
      @response[@oids[:clients]]
    end

    def sys
      @response[@oids[:system]]
    end

    def status_up?(ap_oid)
      return true if @response[@oids[:status]].nil?
      @response[@oids[:status]][@oids[:status] + '.' + ap_oid] == 1
    end

    def parse_mac(mac)
      mac.unpack('H2H2H2H2H2H2').join(':').upcase
    end
  end
end
